"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.addBuildCommands = void 0;
const path_1 = __importDefault(require("path"));
const consts_1 = require("../../consts");
const networks_1 = require("../../networks");
const sui_1 = require("../../sui");
const utils_1 = require("../../utils");
const README_URL = "https://github.com/wormhole-foundation/wormhole/blob/main/sui/README.md";
const addBuildCommands = (y) => y.command("build-coin", `Build wrapped coin and dump bytecode.
    
    Example:
      worm sui build-coin -d 8 -v V__0_1_1 -n testnet -r "https://fullnode.testnet.sui.io:443"`, (yargs) => yargs
    .option("decimals", {
    alias: "d",
    describe: "Decimals of asset",
    required: true,
    type: "number",
})
    // Can't be called version because of a conflict with the native version option
    .option("version-struct", {
    alias: "v",
    describe: "Version control struct name (e.g. V__0_1_0)",
    required: true,
    type: "string",
})
    .option("network", consts_1.NETWORK_OPTIONS)
    .option("package-path", {
    alias: "p",
    describe: "Path to coin module",
    required: false,
    type: "string",
})
    .option("wormhole-state", {
    alias: "w",
    describe: "Wormhole state object ID",
    required: false,
    type: "string",
})
    .option("token-bridge-state", {
    alias: "t",
    describe: "Token bridge state object ID",
    required: false,
    type: "string",
})
    .option("rpc", consts_1.RPC_OPTIONS), async (argv) => {
    var _a, _b, _c, _d;
    (0, utils_1.checkBinary)("sui", README_URL);
    const network = argv.network.toUpperCase();
    (0, utils_1.assertNetwork)(network);
    const decimals = argv["decimals"];
    const version = argv["version-struct"];
    const packagePath = (_a = argv["package-path"]) !== null && _a !== void 0 ? _a : path_1.default.resolve(__dirname, "../../../../../sui/examples");
    const coreBridgeStateObjectId = (_b = argv["wormhole-state"]) !== null && _b !== void 0 ? _b : consts_1.CONTRACTS[network].sui.core;
    const tokenBridgeStateObjectId = (_c = argv["token-bridge-state"]) !== null && _c !== void 0 ? _c : consts_1.CONTRACTS[network].sui.token_bridge;
    const provider = (0, sui_1.getProvider)(network, (_d = argv.rpc) !== null && _d !== void 0 ? _d : networks_1.NETWORKS[network].sui.rpc);
    const build = await (0, sui_1.buildCoin)(provider, network, packagePath, coreBridgeStateObjectId, tokenBridgeStateObjectId, version, decimals);
    console.log(build);
    console.log("Bytecode hex:", Buffer.from(build.modules[0], "base64").toString("hex"));
});
exports.addBuildCommands = addBuildCommands;
