"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.deploy = exports.addDeployCommands = void 0;
const fs_1 = __importDefault(require("fs"));
const consts_1 = require("../../consts");
const networks_1 = require("../../networks");
const sui_1 = require("../../sui");
const utils_1 = require("../../utils");
const README_URL = "https://github.com/wormhole-foundation/wormhole/blob/main/sui/README.md";
const addDeployCommands = (y) => y.command("deploy <package-dir>", "Deploy a Sui package", (yargs) => {
    return yargs
        .positional("package-dir", {
        type: "string",
    })
        .option("network", consts_1.NETWORK_OPTIONS)
        .option("debug", consts_1.DEBUG_OPTIONS)
        .option("private-key", consts_1.PRIVATE_KEY_OPTIONS)
        .option("rpc", consts_1.RPC_OPTIONS);
}, async (argv) => {
    var _a;
    (0, utils_1.checkBinary)("sui", README_URL);
    const packageDir = argv["package-dir"];
    const network = argv.network.toUpperCase();
    (0, utils_1.assertNetwork)(network);
    const debug = (_a = argv.debug) !== null && _a !== void 0 ? _a : false;
    const privateKey = argv["private-key"];
    const rpc = argv.rpc;
    const res = await (0, exports.deploy)(network, packageDir, rpc, privateKey);
    // Dump deployment info to console
    (0, sui_1.logTransactionDigest)(res);
    (0, sui_1.logPublishedPackageId)(res);
    if (debug) {
        (0, sui_1.logTransactionSender)(res);
        (0, sui_1.logCreatedObjects)(res);
    }
});
exports.addDeployCommands = addDeployCommands;
const deploy = async (network, packageDir, rpc, privateKey) => {
    rpc = rpc !== null && rpc !== void 0 ? rpc : networks_1.NETWORKS[network].sui.rpc;
    const provider = (0, sui_1.getProvider)(network, rpc);
    const signer = (0, sui_1.getSigner)(provider, network, privateKey);
    // Allow absolute paths, otherwise assume relative to directory `worm` command is run from
    const dir = packageDir.startsWith("/")
        ? packageDir
        : `${process.cwd()}/${packageDir}`;
    const packagePath = dir.endsWith("/") ? dir.slice(0, -1) : dir;
    if (!fs_1.default.existsSync(packagePath)) {
        throw new Error(`Package directory ${packagePath} does not exist. Make sure to deploy from the correct directory or provide an absolute path.`);
    }
    return (0, sui_1.publishPackage)(signer, network, packagePath);
};
exports.deploy = deploy;
