"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.initWormhole = exports.initTokenBridge = exports.initExampleApp = exports.addInitCommands = void 0;
const sui_js_1 = require("@mysten/sui.js");
const consts_1 = require("../../consts");
const networks_1 = require("../../networks");
const sui_1 = require("../../sui");
const utils_1 = require("../../utils");
const addInitCommands = (y) => y
    .command("init-example-message-app", "Initialize example core message app", (yargs) => {
    return yargs
        .option("network", consts_1.NETWORK_OPTIONS)
        .option("package-id", {
        alias: "p",
        describe: "Example app package ID",
        required: true,
        type: "string",
    })
        .option("wormhole-state", {
        alias: "w",
        describe: "Wormhole state object ID",
        required: true,
        type: "string",
    })
        .option("private-key", consts_1.PRIVATE_KEY_OPTIONS)
        .option("rpc", consts_1.RPC_OPTIONS);
}, async (argv) => {
    const network = argv.network.toUpperCase();
    (0, utils_1.assertNetwork)(network);
    const packageId = argv["package-id"];
    const wormholeStateObjectId = argv["wormhole-state"];
    const privateKey = argv["private-key"];
    const rpc = argv.rpc;
    const res = await (0, exports.initExampleApp)(network, packageId, wormholeStateObjectId, rpc, privateKey);
    (0, sui_1.logTransactionDigest)(res);
    (0, sui_1.logTransactionSender)(res);
    console.log("Example app state object ID", (0, sui_1.getCreatedObjects)(res).find((e) => (0, sui_1.isSameType)(e.type, `${packageId}::sender::State`)).objectId);
})
    .command("init-token-bridge", "Initialize token bridge contract", (yargs) => {
    return yargs
        .option("network", consts_1.NETWORK_OPTIONS)
        .option("package-id", {
        alias: "p",
        describe: "Token bridge package ID",
        required: true,
        type: "string",
    })
        .option("wormhole-state", {
        alias: "w",
        describe: "Wormhole state object ID",
        required: true,
        type: "string",
    })
        .option("governance-chain-id", {
        alias: "c",
        describe: "Governance chain ID",
        default: consts_1.GOVERNANCE_CHAIN,
        type: "number",
        required: false,
    })
        .option("governance-address", {
        alias: "a",
        describe: "Governance contract address",
        type: "string",
        default: consts_1.GOVERNANCE_EMITTER,
        required: false,
    })
        .option("private-key", consts_1.PRIVATE_KEY_OPTIONS)
        .option("rpc", consts_1.RPC_OPTIONS);
}, async (argv) => {
    var _a;
    const network = argv.network.toUpperCase();
    (0, utils_1.assertNetwork)(network);
    const packageId = argv["package-id"];
    const wormholeStateObjectId = argv["wormhole-state"];
    const governanceChainId = argv["governance-chain-id"];
    const governanceContract = argv["governance-address"];
    const privateKey = argv["private-key"];
    const rpc = (_a = argv.rpc) !== null && _a !== void 0 ? _a : networks_1.NETWORKS[network].sui.rpc;
    const res = await (0, exports.initTokenBridge)(network, packageId, wormholeStateObjectId, governanceChainId, governanceContract, rpc, privateKey);
    (0, sui_1.logTransactionDigest)(res);
    (0, sui_1.logTransactionSender)(res);
    console.log("Token bridge state object ID", (0, sui_1.getCreatedObjects)(res).find((e) => (0, sui_1.isSameType)(e.type, `${packageId}::state::State`)).objectId);
})
    .command("init-wormhole", "Initialize wormhole core contract", (yargs) => {
    return yargs
        .option("network", consts_1.NETWORK_OPTIONS)
        .option("package-id", {
        alias: "p",
        describe: "Core bridge package ID",
        required: true,
        type: "string",
    })
        .option("initial-guardian", {
        alias: "i",
        required: true,
        describe: "Initial guardian public keys",
        type: "string",
    })
        .option("debug", consts_1.DEBUG_OPTIONS)
        .option("governance-chain-id", {
        alias: "c",
        describe: "Governance chain ID",
        default: consts_1.GOVERNANCE_CHAIN,
        type: "number",
        required: false,
    })
        .option("guardian-set-index", {
        alias: "s",
        describe: "Governance set index",
        default: 0,
        type: "number",
        required: false,
    })
        .option("governance-address", {
        alias: "a",
        describe: "Governance contract address",
        type: "string",
        default: consts_1.GOVERNANCE_EMITTER,
        required: false,
    })
        .option("private-key", consts_1.PRIVATE_KEY_OPTIONS)
        .option("rpc", consts_1.RPC_OPTIONS);
}, async (argv) => {
    var _a;
    const network = argv.network.toUpperCase();
    (0, utils_1.assertNetwork)(network);
    const packageId = argv["package-id"];
    const initialGuardian = argv["initial-guardian"];
    const debug = (_a = argv.debug) !== null && _a !== void 0 ? _a : false;
    const governanceChainId = argv["governance-chain-id"];
    const guardianSetIndex = argv["guardian-set-index"];
    const governanceContract = argv["governance-address"];
    const privateKey = argv["private-key"];
    const rpc = argv.rpc;
    const res = await (0, exports.initWormhole)(network, packageId, initialGuardian, governanceChainId, guardianSetIndex, governanceContract, rpc, privateKey);
    (0, sui_1.logTransactionDigest)(res);
    console.log("Wormhole state object ID", (0, sui_1.getCreatedObjects)(res).find((e) => (0, sui_1.isSameType)(e.type, `${packageId}::state::State`)).objectId);
    if (debug) {
        (0, sui_1.logTransactionSender)(res);
    }
});
exports.addInitCommands = addInitCommands;
const initExampleApp = async (network, packageId, wormholeStateObjectId, rpc, privateKey) => {
    rpc = rpc !== null && rpc !== void 0 ? rpc : networks_1.NETWORKS[network].sui.rpc;
    const provider = (0, sui_1.getProvider)(network, rpc);
    const signer = (0, sui_1.getSigner)(provider, network, privateKey);
    const transactionBlock = new sui_js_1.TransactionBlock();
    if (network === "DEVNET") {
        // Avoid Error checking transaction input objects: GasBudgetTooHigh { gas_budget: 50000000000, max_budget: 10000000000 }
        transactionBlock.setGasBudget(10000000000);
    }
    transactionBlock.moveCall({
        target: `${packageId}::sender::init_with_params`,
        arguments: [transactionBlock.object(wormholeStateObjectId)],
    });
    return (0, sui_1.executeTransactionBlock)(signer, transactionBlock);
};
exports.initExampleApp = initExampleApp;
const initTokenBridge = async (network, tokenBridgePackageId, coreBridgeStateObjectId, governanceChainId, governanceContract, rpc, privateKey) => {
    rpc = rpc !== null && rpc !== void 0 ? rpc : networks_1.NETWORKS[network].sui.rpc;
    const provider = (0, sui_1.getProvider)(network, rpc);
    const signer = (0, sui_1.getSigner)(provider, network, privateKey);
    const owner = await signer.getAddress();
    const deployerCapObjectId = await (0, sui_1.getOwnedObjectId)(provider, owner, tokenBridgePackageId, "setup", "DeployerCap");
    if (!deployerCapObjectId) {
        throw new Error(`Token bridge cannot be initialized because deployer capability cannot be found under ${owner}. Is the package published?`);
    }
    const upgradeCapObjectId = await (0, sui_1.getUpgradeCapObjectId)(provider, owner, tokenBridgePackageId);
    if (!upgradeCapObjectId) {
        throw new Error(`Token bridge cannot be initialized because upgrade capability cannot be found under ${owner}. Is the package published?`);
    }
    const wormholePackageId = await (0, sui_1.getPackageId)(provider, coreBridgeStateObjectId);
    const transactionBlock = new sui_js_1.TransactionBlock();
    if (network === "DEVNET") {
        // Avoid Error checking transaction input objects: GasBudgetTooHigh { gas_budget: 50000000000, max_budget: 10000000000 }
        transactionBlock.setGasBudget(10000000000);
    }
    const [emitterCap] = transactionBlock.moveCall({
        target: `${wormholePackageId}::emitter::new`,
        arguments: [transactionBlock.object(coreBridgeStateObjectId)],
    });
    transactionBlock.moveCall({
        target: `${tokenBridgePackageId}::setup::complete`,
        arguments: [
            transactionBlock.object(deployerCapObjectId),
            transactionBlock.object(upgradeCapObjectId),
            emitterCap,
            transactionBlock.pure(governanceChainId),
            transactionBlock.pure([...Buffer.from(governanceContract, "hex")]),
        ],
    });
    return (0, sui_1.executeTransactionBlock)(signer, transactionBlock);
};
exports.initTokenBridge = initTokenBridge;
const initWormhole = async (network, coreBridgePackageId, initialGuardians, governanceChainId, guardianSetIndex, governanceContract, rpc, privateKey) => {
    rpc = rpc !== null && rpc !== void 0 ? rpc : networks_1.NETWORKS[network].sui.rpc;
    const provider = (0, sui_1.getProvider)(network, rpc);
    const signer = (0, sui_1.getSigner)(provider, network, privateKey);
    const owner = await signer.getAddress();
    const deployerCapObjectId = await (0, sui_1.getOwnedObjectId)(provider, owner, coreBridgePackageId, "setup", "DeployerCap");
    if (!deployerCapObjectId) {
        throw new Error(`Wormhole cannot be initialized because deployer capability cannot be found under ${owner}. Is the package published?`);
    }
    const upgradeCapObjectId = await (0, sui_1.getUpgradeCapObjectId)(provider, owner, coreBridgePackageId);
    if (!upgradeCapObjectId) {
        throw new Error(`Wormhole cannot be initialized because upgrade capability cannot be found under ${owner}. Is the package published?`);
    }
    const transactionBlock = new sui_js_1.TransactionBlock();
    if (network === "DEVNET") {
        // Avoid Error checking transaction input objects: GasBudgetTooHigh { gas_budget: 50000000000, max_budget: 10000000000 }
        transactionBlock.setGasBudget(10000000000);
    }
    transactionBlock.moveCall({
        target: `${coreBridgePackageId}::setup::complete`,
        arguments: [
            transactionBlock.object(deployerCapObjectId),
            transactionBlock.object(upgradeCapObjectId),
            transactionBlock.pure(governanceChainId),
            transactionBlock.pure([...Buffer.from(governanceContract, "hex")]),
            transactionBlock.pure(guardianSetIndex),
            transactionBlock.pure(initialGuardians.split(",").map((g) => [...Buffer.from(g, "hex")])),
            transactionBlock.pure(24 * 60 * 60),
            transactionBlock.pure("0"), // Message fee
        ],
    });
    return (0, sui_1.executeTransactionBlock)(signer, transactionBlock);
};
exports.initWormhole = initWormhole;
