"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.addPublishMessageCommands = void 0;
const sui_js_1 = require("@mysten/sui.js");
const consts_1 = require("../../consts");
const networks_1 = require("../../networks");
const sui_1 = require("../../sui");
const utils_1 = require("../../utils");
const addPublishMessageCommands = (y) => y.command("publish-example-message", "Publish message from example app via core bridge", (yargs) => {
    return yargs
        .option("network", consts_1.NETWORK_OPTIONS)
        .option("package-id", {
        alias: "p",
        describe: "Package ID/module address",
        required: true,
        type: "string",
    })
        .option("state", {
        alias: "s",
        describe: "Core messages app state object ID",
        required: true,
        type: "string",
    })
        .option("wormhole-state", {
        alias: "w",
        describe: "Wormhole state object ID",
        required: true,
        type: "string",
    })
        .option("message", {
        alias: "m",
        describe: "Message payload",
        required: true,
        type: "string",
    })
        .option("private-key", {
        alias: "k",
        describe: "Custom private key to sign txs",
        required: false,
        type: "string",
    })
        .option("rpc", consts_1.RPC_OPTIONS);
}, async (argv) => {
    var _a;
    const network = argv.network.toUpperCase();
    (0, utils_1.assertNetwork)(network);
    const packageId = argv["package-id"];
    const stateObjectId = argv["state"];
    const wormholeStateObjectId = argv["wormhole-state"];
    const message = argv["message"];
    const privateKey = argv["private-key"];
    const rpc = (_a = argv.rpc) !== null && _a !== void 0 ? _a : networks_1.NETWORKS[network].sui.rpc;
    const provider = (0, sui_1.getProvider)(network, rpc);
    const signer = (0, sui_1.getSigner)(provider, network, privateKey);
    // Publish message
    const transactionBlock = new sui_js_1.TransactionBlock();
    if (network === "DEVNET") {
        // Avoid Error checking transaction input objects: GasBudgetTooHigh { gas_budget: 50000000000, max_budget: 10000000000 }
        transactionBlock.setGasBudget(10000000000);
    }
    transactionBlock.moveCall({
        target: `${packageId}::sender::send_message_entry`,
        arguments: [
            transactionBlock.object(stateObjectId),
            transactionBlock.object(wormholeStateObjectId),
            transactionBlock.pure(message),
            transactionBlock.object(sui_js_1.SUI_CLOCK_OBJECT_ID),
        ],
    });
    const res = await (0, sui_1.executeTransactionBlock)(signer, transactionBlock);
    // Hacky way to grab event since we don't require package ID of the
    // core bridge as input. Doesn't really matter since this is a test
    // command.
    const event = res.events.find((e) => (0, sui_js_1.normalizeSuiAddress)(e.packageId) === (0, sui_js_1.normalizeSuiAddress)(packageId) &&
        e.type.includes("publish_message::WormholeMessage"));
    if (!event) {
        throw new Error("Couldn't find publish event. Events: " +
            JSON.stringify(res.events, null, 2));
    }
    (0, sui_1.logTransactionDigest)(res);
    (0, sui_1.logTransactionSender)(res);
    console.log("Publish message succeeded:", {
        sender: event.sender,
        type: event.type,
        payload: Buffer.from(event.parsedJson.payload).toString(),
        emitter: Buffer.from(event.parsedJson.sender).toString("hex"),
        sequence: event.parsedJson.sequence,
        nonce: event.parsedJson.nonce,
    });
});
exports.addPublishMessageCommands = addPublishMessageCommands;
