"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.addUtilsCommands = void 0;
const consts_1 = require("../../consts");
const networks_1 = require("../../networks");
const sui_1 = require("../../sui");
const utils_1 = require("../../utils");
const addUtilsCommands = (y) => y
    .command("objects <owner>", "Get owned objects by owner", (yargs) => yargs
    .positional("owner", {
    describe: "Owner address",
    type: "string",
})
    .option("network", consts_1.NETWORK_OPTIONS)
    .option("rpc", consts_1.RPC_OPTIONS), async (argv) => {
    var _a;
    const network = argv.network.toUpperCase();
    (0, utils_1.assertNetwork)(network);
    const rpc = (_a = argv.rpc) !== null && _a !== void 0 ? _a : networks_1.NETWORKS[network].sui.rpc;
    const owner = argv.owner;
    const provider = (0, sui_1.getProvider)(network, rpc);
    const objects = [];
    let cursor = undefined;
    while (true) {
        const res = await provider.getOwnedObjects({ owner, cursor });
        objects.push(...res.data);
        if (res.hasNextPage) {
            cursor = res.nextCursor;
        }
        else {
            break;
        }
    }
    console.log("Network", network);
    console.log("Owner", owner);
    console.log("Objects", JSON.stringify(objects, null, 2));
})
    .command("package-id <state-object-id>", "Get package ID from State object ID", (yargs) => yargs
    .positional("state-object-id", {
    describe: "Object ID of State object",
    type: "string",
})
    .option("network", consts_1.NETWORK_OPTIONS)
    .option("rpc", consts_1.RPC_OPTIONS), async (argv) => {
    var _a;
    const network = argv.network.toUpperCase();
    (0, utils_1.assertNetwork)(network);
    const rpc = (_a = argv.rpc) !== null && _a !== void 0 ? _a : networks_1.NETWORKS[network].sui.rpc;
    const provider = (0, sui_1.getProvider)(network, rpc);
    console.log(await (0, sui_1.getPackageId)(provider, argv["state-object-id"]));
})
    // This command is useful for debugging, especially when the Sui explorer
    // goes down :)
    .command("tx <transaction-digest>", "Get transaction details", (yargs) => yargs
    .positional("transaction-digest", {
    describe: "Digest of transaction to fetch",
    type: "string",
})
    .option("network", {
    alias: "n",
    describe: "Network",
    type: "string",
    choices: ["mainnet", "testnet", "devnet"],
    default: "devnet",
    required: false,
})
    .option("rpc", consts_1.RPC_OPTIONS), async (argv) => {
    var _a;
    const network = argv.network.toUpperCase();
    (0, utils_1.assertNetwork)(network);
    const rpc = (_a = argv.rpc) !== null && _a !== void 0 ? _a : networks_1.NETWORKS[network].sui.rpc;
    const provider = (0, sui_1.getProvider)(network, rpc);
    console.log(JSON.stringify(await provider.getTransactionBlock({
        digest: argv["transaction-digest"],
        options: {
            showInput: true,
            showEffects: true,
            showEvents: true,
            showObjectChanges: true,
        },
    }), null, 2));
});
exports.addUtilsCommands = addUtilsCommands;
