"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.execute_sei = void 0;
const proto_signing_1 = require("@cosmjs/proto-signing");
const stargate_1 = require("@cosmjs/stargate");
const core_1 = require("@sei-js/core");
const vaa_1 = require("./vaa");
const networks_1 = require("./networks");
const consts_1 = require("@certusone/wormhole-sdk/lib/cjs/utils/consts");
async function execute_sei(payload, vaa, network) {
    let chain = "sei";
    let n = networks_1.NETWORKS[network][chain];
    let contracts = consts_1.CONTRACTS[network][chain];
    let target_contract;
    let execute_msg;
    switch (payload.module) {
        case "Core":
            target_contract = contracts.core;
            // sigh...
            execute_msg = {
                submit_v_a_a: {
                    vaa: vaa.toString("base64"),
                },
            };
            switch (payload.type) {
                case "GuardianSetUpgrade":
                    console.log("Submitting new guardian set");
                    break;
                case "ContractUpgrade":
                    console.log("Upgrading core contract");
                    break;
                case "RecoverChainId":
                    throw new Error("RecoverChainId not supported on sei");
                default:
                    (0, vaa_1.impossible)(payload);
            }
            break;
        case "NFTBridge":
            if (contracts.nft_bridge === undefined) {
                // NOTE: this code can safely be removed once the sei NFT bridge is
                // released, but it's fine for it to stay, as the condition will just be
                // skipped once 'contracts.nft_bridge' is defined
                throw new Error("NFT bridge not supported yet for sei");
            }
            target_contract = contracts.nft_bridge;
            execute_msg = {
                submit_vaa: {
                    data: vaa.toString("base64"),
                },
            };
            switch (payload.type) {
                case "ContractUpgrade":
                    console.log("Upgrading contract");
                    break;
                case "RecoverChainId":
                    throw new Error("RecoverChainId not supported on sei");
                case "RegisterChain":
                    console.log("Registering chain");
                    break;
                case "Transfer":
                    console.log("Completing transfer");
                    break;
                default:
                    (0, vaa_1.impossible)(payload);
            }
            break;
        case "TokenBridge":
            target_contract = contracts.token_bridge;
            execute_msg = {
                submit_vaa: {
                    data: vaa.toString("base64"),
                },
            };
            switch (payload.type) {
                case "ContractUpgrade":
                    console.log("Upgrading contract");
                    break;
                case "RecoverChainId":
                    throw new Error("RecoverChainId not supported on sei");
                case "RegisterChain":
                    console.log("Registering chain");
                    break;
                case "Transfer":
                    console.log("Completing transfer");
                    break;
                case "AttestMeta":
                    console.log("Creating wrapped token");
                    break;
                case "TransferWithPayload":
                    throw Error("Can't complete payload 3 transfer from CLI");
                default:
                    (0, vaa_1.impossible)(payload);
                    break;
            }
            break;
        default:
            target_contract = (0, vaa_1.impossible)(payload);
            execute_msg = (0, vaa_1.impossible)(payload);
    }
    const wallet = await proto_signing_1.DirectSecp256k1HdWallet.fromMnemonic(n.key, { prefix: "sei" });
    const [account] = await wallet.getAccounts();
    const client = await (0, core_1.getSigningCosmWasmClient)(n.rpc, wallet);
    const fee = (0, stargate_1.calculateFee)(300000, "0.1usei");
    const result = await client.execute(account.address, target_contract, execute_msg, fee);
    console.log(`TX hash: ${result.transactionHash}`);
}
exports.execute_sei = execute_sei;
