"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.execute_solana = void 0;
const web3s = __importStar(require("@solana/web3.js"));
const networks_1 = require("./networks");
const vaa_1 = require("./vaa");
const bs58_1 = __importDefault(require("bs58"));
const solana_1 = require("@certusone/wormhole-sdk/lib/cjs/solana");
const consts_1 = require("@certusone/wormhole-sdk/lib/cjs/utils/consts");
const wormhole_1 = require("@certusone/wormhole-sdk/lib/cjs/solana/wormhole");
const tokenBridge_1 = require("@certusone/wormhole-sdk/lib/cjs/solana/tokenBridge");
const nftBridge_1 = require("@certusone/wormhole-sdk/lib/cjs/solana/nftBridge");
async function execute_solana(v, vaa, network, chain) {
    let ix;
    const connection = setupConnection(networks_1.NETWORKS[network][chain].rpc);
    const bridgeId = new web3s.PublicKey(consts_1.CONTRACTS[network][chain].core);
    const tokenBridgeId = consts_1.CONTRACTS[network][chain].token_bridge &&
        new web3s.PublicKey(consts_1.CONTRACTS[network][chain].token_bridge);
    const nftBridgeId = consts_1.CONTRACTS[network][chain].nft_bridge &&
        new web3s.PublicKey(consts_1.CONTRACTS[network][chain].nft_bridge);
    const from = web3s.Keypair.fromSecretKey(bs58_1.default.decode(networks_1.NETWORKS[network][chain].key));
    switch (v.payload.module) {
        case "Core":
            if (bridgeId === undefined) {
                throw Error("core bridge contract is undefined");
            }
            switch (v.payload.type) {
                case "GuardianSetUpgrade":
                    console.log("Submitting new guardian set");
                    ix = (0, wormhole_1.createUpgradeGuardianSetInstruction)(bridgeId, from.publicKey, vaa);
                    break;
                case "ContractUpgrade":
                    console.log("Upgrading core contract");
                    ix = (0, wormhole_1.createUpgradeContractInstruction)(bridgeId, from.publicKey, vaa);
                    break;
                case "RecoverChainId":
                    throw new Error("RecoverChainId not supported on solana");
                default:
                    ix = (0, vaa_1.impossible)(v.payload);
            }
            break;
        case "NFTBridge":
            if (nftBridgeId === undefined) {
                throw Error("nft bridge contract is undefined");
            }
            switch (v.payload.type) {
                case "ContractUpgrade":
                    console.log("Upgrading contract");
                    ix = (0, nftBridge_1.createUpgradeContractInstruction)(nftBridgeId, bridgeId, from.publicKey, vaa);
                    break;
                case "RecoverChainId":
                    throw new Error("RecoverChainId not supported on solana");
                case "RegisterChain":
                    console.log("Registering chain");
                    ix = (0, nftBridge_1.createRegisterChainInstruction)(nftBridgeId, bridgeId, from.publicKey, vaa);
                    break;
                case "Transfer":
                    throw Error("Can't redeem NFTs from CLI");
                // TODO: what's the authority account? just bail for now
                default:
                    ix = (0, vaa_1.impossible)(v.payload);
            }
            break;
        case "TokenBridge":
            if (tokenBridgeId === undefined) {
                throw Error("token bridge contract is undefined");
            }
            const payload = v.payload;
            switch (payload.type) {
                case "ContractUpgrade":
                    console.log("Upgrading contract");
                    ix = (0, tokenBridge_1.createUpgradeContractInstruction)(tokenBridgeId, bridgeId, from.publicKey, vaa);
                    break;
                case "RecoverChainId":
                    throw new Error("RecoverChainId not supported on solana");
                case "RegisterChain":
                    console.log("Registering chain");
                    ix = (0, tokenBridge_1.createRegisterChainInstruction)(tokenBridgeId, bridgeId, from.publicKey, vaa);
                    break;
                case "Transfer":
                    console.log("Completing transfer");
                    if (payload.tokenChain === consts_1.CHAINS[chain]) {
                        ix = (0, tokenBridge_1.createCompleteTransferNativeInstruction)(tokenBridgeId, bridgeId, from.publicKey, vaa);
                    }
                    else {
                        ix = (0, tokenBridge_1.createCompleteTransferWrappedInstruction)(tokenBridgeId, bridgeId, from.publicKey, vaa);
                    }
                    break;
                case "AttestMeta":
                    console.log("Creating wrapped token");
                    ix = (0, tokenBridge_1.createCreateWrappedInstruction)(tokenBridgeId, bridgeId, from.publicKey, vaa);
                    break;
                case "TransferWithPayload":
                    throw Error("Can't complete payload 3 transfer from CLI");
                default:
                    (0, vaa_1.impossible)(payload);
                    break;
            }
            break;
        default:
            ix = (0, vaa_1.impossible)(v.payload);
    }
    // First upload the VAA
    await (0, solana_1.postVaaSolanaWithRetry)(connection, async (tx) => {
        tx.partialSign(from);
        return tx;
    }, bridgeId, from.publicKey, vaa);
    // Then do the actual thing
    const transaction = new web3s.Transaction().add(ix);
    const signature = await web3s.sendAndConfirmTransaction(connection, transaction, [from], {
        skipPreflight: true,
    });
    console.log("SIGNATURE", signature);
}
exports.execute_solana = execute_solana;
function setupConnection(rpc) {
    return new web3s.Connection(rpc, "confirmed");
}
