"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.execute_terra = void 0;
const terra_js_1 = require("@terra-money/terra.js");
const js_base64_1 = require("js-base64");
const vaa_1 = require("./vaa");
const networks_1 = require("./networks");
const axios_1 = __importDefault(require("axios"));
const consts_1 = require("@certusone/wormhole-sdk/lib/cjs/utils/consts");
async function execute_terra(payload, vaa, network, chain) {
    let n = networks_1.NETWORKS[network][chain];
    let contracts = consts_1.CONTRACTS[network][chain];
    const terra = new terra_js_1.LCDClient({
        URL: n.rpc,
        chainID: n.chain_id,
        isClassic: chain === "terra",
    });
    const wallet = terra.wallet(new terra_js_1.MnemonicKey({
        mnemonic: n.key,
    }));
    let target_contract;
    let execute_msg;
    switch (payload.module) {
        case "Core":
            target_contract = contracts.core;
            // sigh...
            execute_msg = {
                submit_v_a_a: {
                    vaa: (0, js_base64_1.fromUint8Array)(vaa),
                },
            };
            switch (payload.type) {
                case "GuardianSetUpgrade":
                    console.log("Submitting new guardian set");
                    break;
                case "ContractUpgrade":
                    console.log("Upgrading core contract");
                    break;
                case "RecoverChainId":
                    throw new Error("RecoverChainId not supported on terra");
                default:
                    (0, vaa_1.impossible)(payload);
            }
            break;
        case "NFTBridge":
            if (contracts.nft_bridge === undefined) {
                // NOTE: this code can safely be removed once the terra NFT bridge is
                // released, but it's fine for it to stay, as the condition will just be
                // skipped once 'contracts.nft_bridge' is defined
                throw new Error("NFT bridge not supported yet for terra");
            }
            target_contract = contracts.nft_bridge;
            execute_msg = {
                submit_vaa: {
                    data: (0, js_base64_1.fromUint8Array)(vaa),
                },
            };
            switch (payload.type) {
                case "ContractUpgrade":
                    console.log("Upgrading contract");
                    break;
                case "RecoverChainId":
                    throw new Error("RecoverChainId not supported on terra");
                case "RegisterChain":
                    console.log("Registering chain");
                    break;
                case "Transfer":
                    console.log("Completing transfer");
                    break;
                default:
                    (0, vaa_1.impossible)(payload);
            }
            break;
        case "TokenBridge":
            target_contract = contracts.token_bridge;
            execute_msg = {
                submit_vaa: {
                    data: (0, js_base64_1.fromUint8Array)(vaa),
                },
            };
            switch (payload.type) {
                case "ContractUpgrade":
                    console.log("Upgrading contract");
                    break;
                case "RecoverChainId":
                    throw new Error("RecoverChainId not supported on terra");
                case "RegisterChain":
                    console.log("Registering chain");
                    break;
                case "Transfer":
                    console.log("Completing transfer");
                    break;
                case "AttestMeta":
                    console.log("Creating wrapped token");
                    break;
                case "TransferWithPayload":
                    throw Error("Can't complete payload 3 transfer from CLI");
                default:
                    (0, vaa_1.impossible)(payload);
                    break;
            }
            break;
        default:
            target_contract = (0, vaa_1.impossible)(payload);
            execute_msg = (0, vaa_1.impossible)(payload);
    }
    const transaction = new terra_js_1.MsgExecuteContract(wallet.key.accAddress, target_contract, execute_msg, { uluna: 1000 });
    const feeDenoms = ["uluna"];
    const gasPrices = await axios_1.default
        .get("https://terra-classic-fcd.publicnode.com/v1/txs/gas_prices")
        .then((result) => result.data);
    const feeEstimate = await terra.tx.estimateFee([
        {
            sequenceNumber: await wallet.sequence(),
            publicKey: wallet.key.publicKey,
        },
    ], {
        msgs: [transaction],
        memo: "",
        feeDenoms,
        gasPrices,
    });
    wallet
        .createAndSignTx({
        msgs: [transaction],
        memo: "",
        fee: new terra_js_1.Fee(feeEstimate.gas_limit, feeEstimate.amount.add(new terra_js_1.Coin("uluna", 12))),
    })
        .then((tx) => terra.tx.broadcast(tx))
        .then((result) => {
        console.log(result);
        console.log(`TX hash: ${result.txhash}`);
    });
}
exports.execute_terra = execute_terra;
