"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.execute_algorand = void 0;
const networks_1 = require("./networks");
const vaa_1 = require("./vaa");
const algosdk_1 = require("algosdk");
const algorand_1 = require("@certusone/wormhole-sdk/lib/cjs/algorand");
const consts_1 = require("@certusone/wormhole-sdk/lib/cjs/utils/consts");
async function execute_algorand(payload, vaa, environment) {
    const chainName = "algorand";
    let n = networks_1.NETWORKS[environment][chainName];
    if (!n.key) {
        throw Error(`No ${environment} key defined for Algorand`);
    }
    if (!n.rpc) {
        throw Error(`No ${environment} rpc defined for Algorand`);
    }
    let contracts = consts_1.CONTRACTS[environment][chainName];
    console.log("contracts", contracts);
    const ALGORAND_HOST = {
        algodToken: "",
        algodServer: n.rpc,
        algodPort: "",
    };
    if (environment === "DEVNET") {
        ALGORAND_HOST.algodToken =
            "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa";
        ALGORAND_HOST.algodPort = "4001";
    }
    let target_contract;
    switch (payload.module) {
        case "Core":
            target_contract = contracts.core;
            switch (payload.type) {
                case "GuardianSetUpgrade":
                    console.log("Submitting new guardian set");
                    break;
                case "ContractUpgrade":
                    console.log("Upgrading core contract");
                    break;
                case "RecoverChainId":
                    throw new Error("RecoverChainId not supported on algorand");
                default:
                    (0, vaa_1.impossible)(payload);
            }
            break;
        case "NFTBridge":
            if (contracts.nft_bridge === undefined) {
                // NOTE: this code can safely be removed once the algorand NFT bridge is
                // released, but it's fine for it to stay, as the condition will just be
                // skipped once 'contracts.nft_bridge' is defined
                throw new Error("NFT bridge not supported yet for algorand");
            }
            target_contract = contracts.nft_bridge;
            switch (payload.type) {
                case "ContractUpgrade":
                    console.log("Upgrading contract");
                    break;
                case "RecoverChainId":
                    throw new Error("RecoverChainId not supported on algorand");
                case "RegisterChain":
                    console.log("Registering chain");
                    break;
                case "Transfer":
                    console.log("Completing transfer");
                    break;
                default:
                    (0, vaa_1.impossible)(payload);
            }
            break;
        case "TokenBridge":
            if (contracts.token_bridge === undefined) {
                throw new Error("contracts.token_bridge is undefined");
            }
            target_contract = contracts.token_bridge;
            switch (payload.type) {
                case "ContractUpgrade":
                    console.log("Upgrading contract");
                    break;
                case "RecoverChainId":
                    throw new Error("RecoverChainId not supported on algorand");
                case "RegisterChain":
                    console.log("Registering chain");
                    break;
                case "Transfer":
                    console.log("Completing transfer");
                    break;
                case "AttestMeta":
                    console.log("Creating wrapped token");
                    break;
                case "TransferWithPayload":
                    throw Error("Can't complete payload 3 transfer from CLI");
                default:
                    (0, vaa_1.impossible)(payload);
                    break;
            }
            break;
        default:
            target_contract = (0, vaa_1.impossible)(payload);
    }
    const target = BigInt(parseInt(target_contract));
    const CORE_ID = BigInt(parseInt(contracts.core));
    const algodClient = new algosdk_1.Algodv2(ALGORAND_HOST.algodToken, ALGORAND_HOST.algodServer, ALGORAND_HOST.algodPort);
    const algoWallet = (0, algosdk_1.mnemonicToSecretKey)(n.key);
    // Create transaction
    const txs = await (0, algorand_1._submitVAAAlgorand)(algodClient, target, CORE_ID, vaa, algoWallet.addr);
    // Sign and send transaction
    const result = await (0, algorand_1.signSendAndConfirmAlgorand)(algodClient, txs, algoWallet);
    console.log("Confirmed in round:", result["confirmed-round"]);
}
exports.execute_algorand = execute_algorand;
